from sqlalchemy import create_engine
from model_meta import DBSession, metadata, DeclarativeBase, User
from tgext.tagging.model.setup import setup_model

engine = create_engine('sqlite:///:memory:')
setup_model(dict(DeclarativeBase=DeclarativeBase, metadata=metadata, DBSession=DBSession, User=User))

def setup():
    print 'SETTING UP MODEL'
    DBSession.configure(bind=engine)
    metadata.create_all(engine)

def teardown():
    print 'TEARING DOWN MODEL'
    metadata.drop_all(engine)




import inspect
import metadata
import app_auth

def setup_model(models=None):
    if not models:
        models = inspect.currentframe().f_back.f_locals
    
    metadata.DBSession = models['DBSession']
    metadata.DeclarativeBase = models['DeclarativeBase']
    metadata.metadata = models['metadata']
    app_auth.User = models['User']

    from tagging import Tag, Tagging
    return Tag, Tagging


from model import setup_model

try:
    #This is here to make TW2 serve the image
    from tw2.core import Link
    register_delete_image = Link(modname='tgext.tagging', filename='static/delete.png')
except ImportError:
    pass

class PlugTagging(object):
    def __init__(self, app_config):
        self.app_config = app_config

    def plug(self):
        app_config = self.app_config

        app_models = dict(DBSession=app_config.model.DBSession,
                          DeclarativeBase=app_config.model.DeclarativeBase,
                          metadata=app_config.model.metadata,
                          User=app_config.model.User)
        app_config.model.Tag, app_config.model.Tagging = setup_model(app_models)

def plugme(app_config, options):
    app_config.register_hook('startup', PlugTagging(app_config).plug)
    return dict(appid='tgext.tagging', plug_controller=False)


from tw_compat import TW1, TW2, FakeW
import tg

from tgext.tagging.model.sautils import get_primary_key
Tagging = None

if TW2:
    from tw2.core import Param

    class TagList(TW2.Widget):
        template = "genshi:tgext.tagging.templates.taglist"
        resources= [TW2.jquery_js, TW2.JSLink(modname='tgext.tagging', filename='static/tagging.js'),
                    TW2.CSSLink(modname='tgext.tagging', filename='static/tagging.css')]

        editmode = Param(description='Enable Tags removal and adding', default=True)
        tagging_url = Param(description='Url of the tagging controller', default='/tagging')

        def prepare(self):
            global Tagging
            if Tagging is None:
                from tgext.tagging.model.tagging import Tagging

            super(TagList, self).prepare()
            self.tg_url = tg.url
            self.Tagging = Tagging
            if not isinstance(self.value, dict):
                obj = self.value
                self.value = {'target_id': getattr(obj, get_primary_key(obj.__class__)),
                              'tag_list': Tagging.tag_cloud_for_object(obj)}
if not TW2:
    class TagList(TW1.Widget):
        template = "genshi:tgext.tagging.templates.taglist"
        javascript = [TW1.jquery_js, TW1.JSLink(modname='tgext.tagging', filename='static/tagging.js')]
        css = [TW1.CSSLink(modname='tgext.tagging', filename='static/tagging.css')]

        params = {'editmode': 'Enable Tags removal and adding',
                  'tagging_url': 'Url of the tagging controller'}

        editmode = True
        tagging_url = '/tagging'

        def update_params(self, d):
            global Tagging
            if Tagging is None:
                from tgext.tagging.model.tagging import Tagging

            super(TagList, self).update_params(d)
            d['tg_url'] = tg.url
            d['Tagging'] = Tagging
            if not isinstance(d['value'], dict):
                d['value'] = {'target_id': getattr(d['value'], get_primary_key(d['value'].__class__)),
                               'tag_list': Tagging.tag_cloud_for_object(d['value'])}
            d['w'] = FakeW(d)


import inspect
from sqlalchemy.types import Integer
from sqlalchemy.orm import class_mapper

#mostly adapted from sprox

def get_fields(entity):
    if inspect.isfunction(entity):
        entity = entity()
        
    mapper = class_mapper(entity)
    field_names = mapper.c.keys()
    for prop in mapper.iterate_properties:
        try:
            getattr(mapper.c, prop.key)
            field_names.append(prop.key)
        except AttributeError:
            mapper.get_property(prop.key)
            field_names.append(prop.key)

    return field_names

def get_primary_key(entity):
    if inspect.isfunction(entity):
        entity = entity()
        
    mapper = class_mapper(entity)
    for field_name in get_fields(entity):
        try:
            value = getattr(mapper.c, field_name)
        except AttributeError:
            continue
        
        if value.primary_key:
            if not isinstance(value.type, Integer):
                raise LookupError('Primary Key is not an Integer column')
            return value.key

    #should never arrive here, SQLA should not map tables without primary keys
    raise LookupError('No primary keys found')



from tw_compat import TW1, TW2, FakeW
import tg

if TW2:
    from tw2.core import Param

    class TagCloud(TW2.Widget):
        template = "genshi:tgext.tagging.templates.tagcloud"
        resources= [TW2.jquery_js, TW2.JSLink(modname='tgext.tagging', filename='static/tagging.js'),
                    TW2.CSSLink(modname='tgext.tagging', filename='static/tagging.css')]

        tagging_url = Param(description='Url of the tagging controller', default='/tagging')

        def prepare(self):
            super(TagCloud, self).prepare()

            self.tg_url = tg.url
            self.weight_min = 0
            try:
                self.weight_max = max((t[1] for t in self.value))
            except ValueError:
                self.weight_max = 0
            self.weight_range = self.weight_max - self.weight_min
    
if not TW2:
    class TagCloud(TW1.Widget):
        template = "genshi:tgext.tagging.templates.tagcloud"
        javascript = [TW1.jquery_js, TW1.JSLink(modname='tgext.tagging', filename='static/tagging.js')]
        css = [TW1.CSSLink(modname='tgext.tagging', filename='static/tagging.css')]
    
        params = {'tagging_url': 'Url of the tagging controller'}
    
        tagging_url = '/tagging'
    
        def update_params(self, d):
            super(TagCloud, self).update_params(d)
            d['tg_url'] = tg.url
            d['weight_min'] = 0
            try:
                d['weight_max'] = max((t[1] for t in d['value']))
            except ValueError:
                d['weight_max'] = 0
            d['weight_range'] = d['weight_max'] - d['weight_min']
            d['w'] = FakeW(d)
    
            
        
        



try:
    from tw.api import JSLink as Tw1JSLink,\
                      CSSLink as Tw1CSSLink, \
                       Widget as Tw1Widget
    class TW1:
        JSLink = Tw1JSLink
        CSSLink = Tw1CSSLink
        Widget = Tw1Widget
except ImportError:
    TW1 = None


try:
    from tw2.core.resources import JSLink as Tw2JSLink, \
                                  CSSLink as Tw2CSSLink
    from tw2.core import Widget as Tw2Widget
    class TW2:
        JSLink = Tw2JSLink
        CSSLink = Tw2CSSLink
        Widget = Tw2Widget
except ImportError:
    TW2 = None

if TW2:
    from tw2.jquery import jquery_js as tw2_jquery_js
    TW2.jquery_js = tw2_jquery_js

if TW1 and not TW2:
    from tw.jquery import jquery_js as tw1_jquery_js
    TW1.jquery_js = tw1_jquery_js

def is_tw2_form(w):
    return hasattr(w, 'req')

class FakeW(dict):
    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError()

    @property
    def compound_id(self):
        return self['id']



from tg import expose, flash, require, url, request, redirect
from tg import tmpl_context, validate, config, TGController

try:
    from tg import predicates
    from tg.predicates import NotAuthorizedError
except ImportError:
    from repoze.what import predicates
    from repoze.what.predicates import NotAuthorizedError

def can_edit(predicate):
    if not predicate:
        return True
        
    try:
        predicate.check_authorization(request.environ)
    except NotAuthorizedError, e:
        return False
    return True

def get_tagging_base_url(request, method):
    pos = request.path.rfind(method)-1
    return request.path[:pos]

Tagging = None

class TaggingController(TGController):
    def __init__(self, session, model, allow_edit=None):
        super(TaggingController, self).__init__()
        self.session = session
        self.model = model
        self.allow_edit = allow_edit

    @expose(content_type='text/plain')
    def add(self, target_id, tags, *args, **kw):
        global Tagging
        if Tagging is None:
            from tgext.tagging.model.tagging import Tagging

        if not can_edit(self.allow_edit):
            return 'DENIED'

        obj = self.session.query(self.model).get(target_id)
        Tagging.add_tags(obj, tags)
        return 'OK'

    @expose(content_type='text/plain')
    def remove(self, target_id, tags, *args, **kw):
        global Tagging
        if Tagging is None:
            from tgext.tagging.model.tagging import Tagging

        if not can_edit(self.allow_edit):
            return 'DENIED'

        obj = self.session.query(self.model).get(target_id)
        Tagging.del_tags(obj, tags)
        return 'OK'

    @expose('genshi:tgext.tagging.templates.tags')
    def tags(self, target_id):
        global Tagging
        if Tagging is None:
            from tgext.tagging.model.tagging import Tagging

        obj = self.session.query(self.model).get(target_id)
        value = {}
        value['tag_list'] = Tagging.tag_cloud_for_object(obj)
        value['target_id'] = target_id

        editmode = can_edit(self.allow_edit)
        base_url = get_tagging_base_url(request, 'tags')
        return dict(value=value, tagging_url=base_url, editmode=editmode)

    @expose('genshi:tgext.tagging.templates.search')
    def search(self, tags):
        global Tagging
        if Tagging is None:
            from tgext.tagging.model.tagging import Tagging

        items = Tagging.items_for_tags(self.model, tags)
        return dict(items=items, tags=tags)